@@ -36,7 +36,7 @@
 
 */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -59,9 +59,9 @@ static bool authgss_validate(AUTH *auth, struct opaque_auth *verf);
 static void authgss_destroy(AUTH *auth);
 static void authgss_destroy_context(AUTH *auth);
 static bool authgss_wrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func,
-			 caddr_t xdr_ptr);
+			 void *xdr_ptr);
 static bool authgss_unwrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func,
-			   caddr_t xdr_ptr);
+			   void *xdr_ptr);
 
 /*
  * from mit-krb5-1.2.1 mechglue/mglueP.h:
@@ -574,7 +574,7 @@ authgss_destroy_context(AUTH *auth)
 				      (xdrproc_t) xdr_void, NULL,
 				      (xdrproc_t) xdr_void, NULL);
 			if (clnt_req_setup(cc, to) == RPC_SUCCESS) {
-				CLNT_CALL_WAIT(cc);
+				(void)CLNT_CALL_WAIT(cc);
 			}
 			clnt_req_release(cc);
 		}
@@ -616,11 +616,17 @@ authgss_destroy(AUTH *auth)
 }
 
 bool
-authgss_wrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func, caddr_t xdr_ptr)
+authgss_wrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func, void *xdr_ptr)
 {
 	struct rpc_gss_data *gd = AUTH_PRIVATE(auth);
 
-	__warnx(TIRPC_DEBUG_FLAG_RPCSEC_GSS, "%s()", __func__);
+	__warnx(TIRPC_DEBUG_FLAG_RPCSEC_GSS, "%s() %d %s", __func__,
+		!gd->established ? 0 : gd->sec.svc,
+		!gd->established ? "not established"
+		: gd->sec.svc == RPCSEC_GSS_SVC_NONE ? "krb5"
+		: gd->sec.svc == RPCSEC_GSS_SVC_INTEGRITY ? "krb5i"
+		: gd->sec.svc == RPCSEC_GSS_SVC_PRIVACY ? "krb5p"
+		: "unknown");
 
 	if (!gd->established || gd->sec.svc == RPCSEC_GSS_SVC_NONE)
 		return ((*xdr_func) (xdrs, xdr_ptr));
@@ -631,7 +637,7 @@ authgss_wrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func, caddr_t xdr_ptr)
 }
 
 bool
-authgss_unwrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func, caddr_t xdr_ptr)
+authgss_unwrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func, void *xdr_ptr)
 {
 	struct rpc_gss_data *gd = AUTH_PRIVATE(auth);
 
@@ -642,5 +648,5 @@ authgss_unwrap(AUTH *auth, XDR *xdrs, xdrproc_t xdr_func, caddr_t xdr_ptr)
 
 	return (xdr_rpc_gss_unwrap
 		(xdrs, xdr_func, xdr_ptr, gd->ctx, gd->sec.qop, gd->sec.svc,
-		 gd->gc.gc_seq));
+		 gd->gc.gc_seq, NULL, NULL));
 }
