@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <config.h>
+#include "config.h"
 #include <sys/cdefs.h>
 
 /*
@@ -334,8 +334,7 @@ marshal_new_auth(AUTH *auth)
 }
 
 static bool
-authunix_wrap(AUTH *auth, XDR *xdrs, xdrproc_t xfunc,
-	      caddr_t xwhere)
+authunix_wrap(AUTH *auth, XDR *xdrs, xdrproc_t xfunc, void *xwhere)
 {
 	return ((*xfunc) (xdrs, xwhere));
 }
