@@ -51,6 +51,7 @@
 #include <unistd.h>	//fcntl
 #include <fcntl.h>	//fcntl
 #include <sys/epoll.h>
+#include <urcu-bp.h>
 
 #define EPOLL_SIZE (10)
 /*^ expected number of fd, must be > 0 */
@@ -571,6 +572,7 @@ rpc_rdma_stats_thread(void *arg)
 	int n;
 	int rc;
 
+	rcu_register_thread();
 	while (rpc_rdma_state.run_count > 0) {
 		n = epoll_wait(rpc_rdma_state.stats_epollfd,
 				epoll_events, EPOLL_EVENTS, EPOLL_WAIT_MS);
@@ -634,7 +636,7 @@ rpc_rdma_stats_thread(void *arg)
 			rc = close(childfd);
 		}
 	}
-
+	rcu_unregister_thread();
 	pthread_exit(NULL);
 }
 
@@ -836,6 +838,7 @@ rpc_rdma_cq_thread(void *arg)
 	int n;
 	int rc;
 
+	rcu_register_thread();
 	while (rpc_rdma_state.run_count > 0) {
 		n = epoll_wait(rpc_rdma_state.cq_epollfd,
 				epoll_events, EPOLL_EVENTS, EPOLL_WAIT_MS);
@@ -895,7 +898,7 @@ rpc_rdma_cq_thread(void *arg)
 			mutex_unlock(&xprt->cm_lock);
 		}
 	}
-
+	rcu_unregister_thread();
 	pthread_exit(NULL);
 }
 
@@ -1091,6 +1094,7 @@ rpc_rdma_cm_thread(void *nullarg)
 	int n;
 	int rc;
 
+	rcu_register_thread();
 	while (rpc_rdma_state.run_count > 0) {
 		n = epoll_wait(rpc_rdma_state.cm_epollfd,
 				epoll_events, EPOLL_EVENTS, EPOLL_WAIT_MS);
@@ -1166,7 +1170,7 @@ rpc_rdma_cm_thread(void *nullarg)
 				SVC_DESTROY(&cm_xprt->sm_dr.xprt);
 		}
 	}
-
+	rcu_unregister_thread();
 	pthread_exit(NULL);
 }
 
