@@ -271,7 +271,7 @@ svc_dg_rendezvous(SVCXPRT *xprt)
 		return (XPRT_DIED);
 	}
 
-	if (unlikely(svc_rqst_rearm_events(xprt))) {
+	if (unlikely(svc_rqst_rearm_events(xprt, SVC_XPRT_FLAG_ADDED_RECV))) {
 		__warnx(TIRPC_DEBUG_FLAG_ERROR,
 			"%s: %p fd %d svc_rqst_rearm_events failed (will set dead)",
 			__func__, xprt, xprt->xp_fd);
@@ -312,7 +312,7 @@ svc_dg_recv(SVCXPRT *xprt)
 	/* pass the xdrs to user to store in struct svc_req, as most of
 	 * the work has already been done on rendezvous
 	 */
-	stat = __svc_params->request_cb(xprt, REC_XPRT(xprt)->ioq.xdrs);
+	stat = svc_request(xprt, REC_XPRT(xprt)->ioq.xdrs);
 
 	if (xprt->xp_flags & SVC_XPRT_FLAG_DESTROYED)
 		return (XPRT_DESTROYED);
